# Add a Database Model
Use `sequelize-cli` to generate both the database tables and JavaScript classes.

## Generate a model with model:generate

```
$ npx sequelize model:generate --name Post --attributes title:string,content:text,publishDate:date
```

model:generate creates two files
* models/post.js
* db/migrations/[timestamp]-create-post.js

You must modify both files to reflect the data attributes, constraints, indexes, and other details

---

## Migrate the Database

```
$ npx sequelize db:migrate
```

db:migrate command executes the following steps:
* Ensures a table called SequelizeMeta is in database. This table is used to record which migrations have run on the database.
* Runs any migration files which haven't run yet. This is possible by checking SequelizeMeta table.

In this case, the `create-post` migration file will be executed against the database resulting in a new `posts` table being created.

## Undo a migration

```
$ npx sequelize db:migrate:undo
```

Undoing a migration runs the `down` function of most recently applied migration file.

---

## Seed the database
To "seed" the database is to programmatically insert values into the database -- no user interaction required. This is useful for transferring a known data set into the database or populating tables with faux data for development.

```
# generate the seed
$ npx sequelize seed:generate --name first-posts

### MODIFY THE SEED FILE AT THIS POINT (see below)

# seed the database, i.e. insert records into the database
$ npx sequelize-cli db:seed:all
```

seed:generate creates a file, `db/seeds/[timestamp]-first-posts.js`. Like a migration, the seed file implements an `up/down` interface. The `up` command specifies what actions should be performed to seed the database. The `down` function should specify how to undo the actions.  
db:seed:all runs all seed files not yet run

In the `up` method, use the provided `queryInterface` class to bulk insert the posts into the database.  
In the `down` method, perform the reverse action of the `up` method by deleting the posts. To use Sequelize's `Op` (short for "operation") library, import it at the top of the file.

```javascript
const posts = [{
    title: 'Hello World',
    content: 'This is the first post!',
    publishDate: new Date('2022-01-01'),
    createdAt: new Date(),
    updatedAt: new Date()
}];

module.exports = {
    async up (queryInterface, Sequelize) {
        await queryInterface.bulkInsert('Posts', posts, {});
    },

    async down (queryInterface, Sequelize) {
        // DELETE FROM "Posts" WHERE posts.title IN ["Hello World"];
        await queryInterface.bulkDelete('Posts', {
            title: {
                [Op.in]: posts.map((post) => post.title)
            }
        }, {});
    }
}
```

---

## A note on configuration
By default, every time the `db:seed:all` command is run, the database will be seeded with all seed files.
To keep record of which seeds have been run and not re-run previously recorded seeds,
the configuration `"seederStorage": "sequelize"` is added to the development object of `config/sequelize.js`.

```
module.exports = {
  "development": {
    ...,
    "seederStorage": "sequelize"
  },
  ...
}
```

## Resources

Sequelize model basics: [https://sequelize.org/docs/v6/core-concepts/model-basics](https://sequelize.org/docs/v6/core-concepts/model-basics)

Sequelize Seeds: [https://sequelize.org/docs/v6/other-topics/migrations](https://sequelize.org/docs/v6/other-topics/migrations/#creating-the-first-seed)

Sequelize Query Interface: [https://sequelize.org/docs/v6/other-topics/query-interface/](https://sequelize.org/docs/v6/other-topics/query-interface/)

Sequelize `QueryInterface` API : [https://sequelize.org/api/v6/class/src/dialects/abstract/query-interface.js](https://sequelize.org/api/v6/class/src/dialects/abstract/query-interface.js~queryinterface)